import pygame, sys, random, math
from pygame.locals import *
from pygame.color import *
import pymunk
from pymunk import Vec2d

#HEY HEY HEY, STAY OUTTA' MY SOURCE CODE.

# set up pygame
pygame.mixer.init()
pygame.mixer.pre_init(44100, -16, 2, 2048)
pygame.init()

#-------------------------
# START CONFIG
#-------------------------
# set up the window #Width, Height, Advanced, Color Depth
windowSurface = pygame.display.set_mode((0, 0),  pygame.FULLSCREEN , 32)
WIDTH = windowSurface.get_width()
HEIGHT = windowSurface.get_height()

#COLOURS
BLACK = (0, 0, 0)
WHITE = (255, 255, 255)

#Center Screen
CENTER = 75

#Jump Speed
JUMP_SPEED = 10

#GRAVITY
GRAVITY = -900

#Coefficent of friction against the surface
FRICTION = 0.5

#Frames Per Second
FPS = 50

#Title
pygame.display.set_caption('Chaos Simulator 5000')

#-------------------------
# END CONFIG
#-------------------------

#Small hack because pymunk does not like pygame's y-axis
def flip(y):
    return -y+HEIGHT

#Space to work in
space = pymunk.Space()
space.gravity = Vec2d(0.0, GRAVITY)

#Keeping frames stable
mainClock = pygame.time.Clock()
	
#Boundaries - Static Lines
static_body = pymunk.Body()
static_lines = [pymunk.Segment(static_body, (flip(0), flip(HEIGHT)), (flip(0), flip(HEIGHT)), 0.0),]
for l in static_lines:
	l.friction = FRICTION
space.add(static_lines)

#Render the boundries
for line in static_lines:
	body = line.body
	pv1 = body.position + line.a.rotated(body.angle)
	pv2 = body.position + line.b.rotated(body.angle)
	p1 = pv1.x, flip(pv1.y)
	p2 = pv2.x, flip(pv2.y)
	pygame.draw.lines(windowSurface, BLACK, False, [p1,p2], 2)


#Tossable Objects
class Toss:
	def __init__(self, image, x, y, speedx, speedy):
		self.image = image
		self.pos = image.get_rect().move(x, y)
		self.speedy = speedy
		self.speedx = speedx
		self.airtime = 0
	
	def move(self):
		if self.pos.bottom < (HEIGHT-CENTER):
			self.airtime += (1/100)
			dist = int(((1/2) * 9.81 * (self.airtime**2)) + self.speedy)
			if (self.pos.bottom + dist) > HEIGHT:
				pass
			self.pos = self.pos.move(self.speedx, dist)
			if self.pos.left > WIDTH:
				self.pos.right = 0
			if self.pos.right < 0:
				self.pos.left = WIDTH

class Player:
	def __init__(self, image, x, y):
		self.image = image
		self.pos = image.get_rect().move(x, y)
		self.speed = 0
		self.speedy = 0
		self.jump = False
		self.airtime = 0
		self.direction = "Left"
	
	def move(self):
		
		if self.speed > 0 and self.direction == "Left":
			self.direction = "Right"
			self.image = pygame.transform.flip(self.image, True, False)
		if self.speed < 0 and self.direction == "Right":
			self.direction = "Left"
			self.image = pygame.transform.flip(self.image, True, False)
		if self.pos.left > WIDTH:
			self.pos.right = 0
		if self.pos.right < 0:
			self.pos.left = WIDTH
		dist = 0
		if self.jump == True or (self.pos.centery) < (HEIGHT - CENTER):
			self.airtime += 0.05
			if self.speedy < JUMP_SPEED:
				self.speedy += 1
			dist = int(((1/2) * 9.81 * (self.airtime**2)) - self.speedy)
		else:
			self.airtime = 0
			self.speedy = 0
		if (self.pos.centery + dist) > (HEIGHT - CENTER):
			dist = HEIGHT - self.pos.centery - CENTER
		self.pos = self.pos.move(self.speed, dist)
			

def add_ball(space):
    mass = 1
    radius = 14
    inertia = pymunk.moment_for_circle(mass, 0, radius) # 1
    body = pymunk.Body(mass, inertia) # 2
    x = random.randint(120,380)
    body.position = x, 550 # 3
    shape = pymunk.Circle(body, radius) # 4
    space.add(body, shape) # 5
    return shape

def draw_ball(screen, ball):
    p = int(ball.body.position.x), 600-int(ball.body.position.y)
    pygame.draw.circle(screen, BLACK, p, int(ball.radius), 2)


def draw_lines(screen, lines):
    for line in lines:
        body = line.body
        pv1 = body.position + line.a.rotated(body.angle) # 1
        pv2 = body.position + line.b.rotated(body.angle)
        p1 = to_pygame(pv1) # 2
        p2 = to_pygame(pv2)
        pygame.draw.lines(screen, BLACK, False, [p1,p2])



#Theme music
pygame.mixer.music.load("Music/Menu/loop.mid")
pygame.mixer.music.play(-1, 0)

#Load up the background
backgroundImage = pygame.image.load('Art/Backgrounds/Main_Background.png').convert_alpha()
backgroundImageS = pygame.transform.scale(backgroundImage, (WIDTH, HEIGHT))

#Text of doom
chaosText = pygame.image.load('Art/Text/Chaos_Cloud.png').convert_alpha()
chaosRec = chaosText.get_rect()
chaosTextS = pygame.transform.smoothscale(chaosText, (int(chaosRec.right * 2), int(chaosRec.bottom * 2)))

#Text of doom 2
simText = pygame.image.load('Art/Text/Simulator_5000.png').convert_alpha()
simRec = simText.get_rect()
simTextS = pygame.transform.smoothscale(simText, (int(simRec.right * 2), int(simRec.bottom * 2)))

#Cloud of doom
cloud = pygame.image.load('Art/Objects/Chocolate_Rain_Cloud.png').convert_alpha()
cloudRec = cloud.get_rect()
cloudS = pygame.transform.smoothscale(cloud, (int(chaosRec.width/1.5), int(chaosRec.height * 2)))
cloudMove = 0
cloudMove2 = 0
cloudMove3 = 0
cloudHeight2 = random.randint(1, HEIGHT-20)	
cloudHeight3 = random.randint(1, HEIGHT-20)	

#Start Button
startButton = pygame.image.load('Art/Objects/Start_Button.png').convert_alpha()
startButtonRec = startButton.get_rect()
startButtonRec = startButtonRec.move(-40,int(HEIGHT-50))

#Game Menu
# 0 -> Main Title
# 1 -> Game Mode
GAMEMENU = {'mode' : 0, 'init' : 0}

#-------------------------------
# Main menu game loop for pygame
#-------------------------------
while GAMEMENU['mode'] == 0:
    # check for events
	for event in pygame.event.get():
		#Default
		if event.type == QUIT:
			pygame.quit()
			sys.exit()
		#Key down event
		if event.type == KEYDOWN:
			#Exit Key = Esc
			if event.key == K_ESCAPE:
				pygame.quit()
				sys.exit()
		if event.type == MOUSEBUTTONUP:
			if startButtonRec.collidepoint(event.pos[0], event.pos[1]):
				GAMEMENU['mode'] = 1
				GAMEMENU['init'] = 0
	#--------------------
	# DONE EVENT LOOP
	#--------------------
	
	# DEFAULT: Black Background
	windowSurface.fill(BLACK)
		
	#Move the cloud
	if cloudMove <(int((WIDTH/2) - chaosRec.right + 200)):
		cloudMove += 2
	cloudMove2 += 1
	if cloudMove2 > WIDTH:
		cloudMove2 = 0
		cloudHeight2 = random.randint(1, HEIGHT-20)		
	cloudMove3 += 3
	if cloudMove3 > WIDTH:
		cloudMove3 = 0
		cloudHeight3 = random.randint(1, HEIGHT-20)
			
	# Draw what we have onto the surface
	windowSurface.blit(backgroundImageS, (0,0))
	windowSurface.blit(cloudS,(cloudMove,200))
	windowSurface.blit(cloudS,(cloudMove2-50,cloudHeight2))
	windowSurface.blit(cloudS,(cloudMove3-50,cloudHeight3))
	windowSurface.blit(chaosTextS, (int((WIDTH/2) - chaosRec.right),200))
	windowSurface.blit(simTextS, (int((WIDTH/2) - simRec.right),300))
	windowSurface.blit(startButton, (-40, int(HEIGHT-50)))
		
	# draw the window onto the screen
	pygame.display.update()
		
	#100 fps
	mainClock.tick(FPS)

#Setup for main game!
if GAMEMENU['init'] == 0 and GAMEMENU['mode'] == 1:
	pygame.mixer.music.stop()
	
	sounds = []
	pygame.mixer.music.load("Music/Menu/loop2.mid")
	pygame.mixer.music.play(-1, 0)
	sounds.append(pygame.mixer.Sound('Voices/Chaos_Start.ogg'))
	for sound in sounds:
		sound.play()
	
	#Poor Twilight
	twilight = pygame.image.load('Art/Characters/Twilight/twilight_sparkle.png').convert_alpha()
	player = Player(twilight,int(WIDTH/2), int(HEIGHT-CENTER))
	
	#Parasprites!
	parasprite = pygame.image.load('Art/Characters/Twilight/twilight_sparkle.png').convert_alpha()
	paraspriteRec = parasprite.get_rect()
	paraspriteS =  pygame.transform.smoothscale(parasprite, (int(paraspriteRec.height/15), int(paraspriteRec.width/15)))
	tossers = []
	
	 
	for i in range(1, 2):
		x = random.randint(20,400)
		y = 500
		angle = random.random() * math.pi
		vs = [(-23,26), (23,26), (0,-26)]
		mass = 10
		moment = pymunk.moment_for_poly(mass, vs)
		body = pymunk.Body(mass, moment)
		shape = pymunk.Poly(body, vs)
		shape.friction = 0.5
		body.position = x, y
		body.angle = angle
		space.add(body, shape)
		tossers.append(shape)
	
	GAMEMENU['init'] = 1

#-------------------------------
# Main game loop for pygame
#-------------------------------
while (GAMEMENU['mode']) == 1:
    # check for events
	for event in pygame.event.get():
		#Default
		if event.type == QUIT:
			pygame.quit()
			sys.exit()
		#Key down event
		if event.type == KEYDOWN:
			#Exit Key = Esc
			if event.key == K_ESCAPE:
				pygame.quit()
				sys.exit()
			if event.key == K_LEFT or event.key == ord('a'):
				player.speed = -3
			if event.key == K_RIGHT or event.key == ord('d'):
				player.speed = 3			
			if event.key == K_UP or event.key == ord('w'):
				player.jump = True
		if event.type == KEYUP:
			if event.key == K_LEFT or event.key == ord('a'):
				player.speed = 0
			if event.key == K_RIGHT or event.key == ord('d'):
				player.speed = 0
			if event.key == K_UP or event.key == ord('w'):
				player.jump = False
	#--------------------
	# DONE EVENT LOOP
	#--------------------
	# Update physics
		dt = 1.0/60.0
		for x in range(1):
			space.step(dt)
          
	#White background now
	windowSurface.fill(WHITE)
	
	#Move the player
	player.move()
	
	#Draw our surfaces
	windowSurface.blit(player.image, (player.pos.left, player.pos.centery))
	
	#Move the tossers
	for toss in tossers:
		 p = toss.body.position
		 p = Vec2d(p.x, flip(p.y))
		 
		 # We need to rotate 180 degrees because of the y coordinate flip
		 angle_degrees = math.degrees(toss.body.angle) + 180
		 rotated_toss_img = pygame.transform.rotate(paraspriteS, angle_degrees)
		 
		 offset = Vec2d(rotated_toss_img.get_size()) / 2.
		 p = p - offset
		 windowSurface.blit(rotated_toss_img, p)
		
	
	# draw the window onto the screen
	pygame.display.update()	
	
	#FPS
	mainClock.tick(FPS)
